#!/usr/bin/env python3

############################################################################
#
# MODULE:       lib with tiling related helper functions for GRASS GIS
#
# AUTHOR(S):    Anika Weinmann and Lina Krisztian
#
# PURPOSE:      lib with tiling related helper functions for GRASS GIS
#
# COPYRIGHT:	(C) 2023 by mundialis and the GRASS Development Team
#
# 		This program is free software under the GNU General Public
# 		License (>=v2). Read the file COPYING that comes with GRASS
# 		for details.
#
#############################################################################

import grass.script as grass

from .cleanup import rm_vects, reset_region


def create_grid(tile_size, grid_prefix, sid, area=None):
    """Create a grid for parallelization
    Args:
        tile_size (float): the size for the tiles in map units
        grid_prefix (str): the prefix name for the output grid
        area (str): the name of area for which to create the grid tiles
        sid (str): unique identifier
    Return:
        grid_prefix (list): list with the names of the created vector map tiles
        number_tiles (int): Number of created tiles
    """
    # save region
    orig_region = f"grid_region_{sid}"
    grass.run_command("g.region", save=orig_region, quiet=True)

    if area:
        # set region to area
        grass.run_command("g.region", vector=area, quiet=True)

    # check if region is smaller than tile size
    region = grass.region()
    dist_ns = abs(region["n"] - region["s"])
    dist_ew = abs(region["w"] - region["e"])

    grass.message(_("Creating tiles..."))
    grid = f"tmp_grid_{sid}"
    if dist_ns <= float(tile_size) and dist_ew <= float(tile_size):
        grass.run_command("v.in.region", output=grid, quiet=True)
        grass.run_command(
            "v.db.addtable", map=grid, columns="cat int", quiet=True
        )
    else:
        # set region with tile_size
        grass.run_command("g.region", res=tile_size, flags="a", quiet=True)

        # create grid
        grass.run_command(
            "v.mkgrid", map=grid, box=f"{tile_size},{tile_size}", quiet=True
        )
    # reset region
    reset_region(orig_region)

    grid_name = f"tmp_grid_area_{sid}"
    if area:
        grass.run_command(
            "v.select",
            ainput=grid,
            binput=area,
            output=grid_name,
            operator="overlap",
            quiet=True,
        )
        if grass.find_file(name=grid_name, element="vector")["file"] == "":
            grass.fatal(
                _(
                    f"The set region is not overlapping with {area}. "
                    f"Please define another region."
                )
            )
    else:
        grid_name = grid

    # create list of tiles
    tiles_num_list = list(
        grass.parse_command(
            "v.db.select", map=grid_name, columns="cat", flags="c", quiet=True
        ).keys()
    )

    number_tiles = len(tiles_num_list)
    grass.message(_(f"Number of tiles is: {number_tiles}"))
    tiles_list = []
    for tile in tiles_num_list:
        tile_area = f"{grid_prefix}_{tile}"
        grass.run_command(
            "v.extract",
            input=grid_name,
            where=f"cat == {tile}",
            output=tile_area,
            quiet=True,
            overwrite=True,
        )
        tiles_list.append(tile_area)

    # cleanup
    rm_vects([grid, grid_name])

    return tiles_list



#!/usr/bin/env python3

############################################################################
#
# MODULE:       lib with mapset related helper functions for GRASS GIS
#
# AUTHOR(S):    Anika Weinmann
#
# PURPOSE:      lib with mapset related helper functions for GRASS GIS
#
# COPYRIGHT:	(C) 2023 by mundialis and the GRASS Development Team
#
# 		This program is free software under the GNU General Public
# 		License (>=v2). Read the file COPYING that comes with GRASS
# 		for details.
#
#############################################################################

import os
import shutil

import grass.script as grass


def switch_to_new_mapset(new_mapset):
    """The function switches to a new mapset and changes the GISRC file for
    parallel processing.

    Args:
        new_mapset (string): Unique name of the new mapset
    Returns:
        gisrc (string): The path of the old GISRC file
        newgisrc (string): The path of the new GISRC file
        old_mapset (string): The name of the old mapset
    """
    # current gisdbase, location
    env = grass.gisenv()
    gisdbase = env["GISDBASE"]
    location = env["LOCATION_NAME"]
    old_mapset = env["MAPSET"]

    grass.message(_(f"New mapset {new_mapset}"))
    grass.utils.try_rmdir(os.path.join(gisdbase, location, new_mapset))

    gisrc = os.environ["GISRC"]
    newgisrc = f"{gisrc}_{os.getpid()}"
    grass.try_remove(newgisrc)
    shutil.copyfile(gisrc, newgisrc)
    os.environ["GISRC"] = newgisrc

    grass.message(f"GISRC: {os.environ['GISRC']}")
    grass.run_command("g.mapset", flags="c", mapset=new_mapset)

    # verify that switching of the mapset worked
    cur_mapset = grass.gisenv()["MAPSET"]
    if cur_mapset != new_mapset:
        grass.fatal(f"new mapset is {cur_mapset}, but should be {new_mapset}")
    return gisrc, newgisrc, old_mapset


def verify_mapsets(start_cur_mapset):
    """The function verifies the switches to the start_cur_mapset.

    Args:
        start_cur_mapset (string): Name of the mapset which is to verify
    Returns:
        location_path (string): The path of the location
    """
    env = grass.gisenv()
    gisdbase = env["GISDBASE"]
    location = env["LOCATION_NAME"]
    cur_mapset = env["MAPSET"]
    if cur_mapset != start_cur_mapset:
        grass.fatal(
            f"new mapset is {cur_mapset}, but should be {start_cur_mapset}"
        )
    location_path = os.path.join(gisdbase, location)
    return location_path



#!/usr/bin/env python3

############################################################################
#
# MODULE:       lib with location related helper functions for GRASS GIS
#
# AUTHOR(S):    Lina Krisztian, Anika Weinmann
#
# PURPOSE:      lib with location related helper functions for GRASS GIS
#
# COPYRIGHT:	(C) 2023 by mundialis and the GRASS Development Team
#
# 		This program is free software under the GNU General Public
# 		License (>=v2). Read the file COPYING that comes with GRASS
# 		for details.
#
#############################################################################

import os
import subprocess

import grass.script as grass


def get_location_size():
    """Log size of current location"""
    current_gisdbase = grass.gisenv()["GISDBASE"]
    cmd = grass.Popen(
        f"df -h {current_gisdbase}", shell=True, stdout=subprocess.PIPE
    )
    grass.message(
        _(
            (
                "\nDisk usage of GRASS GIS database:\n",
                f"{cmd.communicate()[0].decode('utf-8').rstrip()}\n",
            ),
        )
    )


def create_tmp_location(epsg=4326):
    """Creation of a new temporary location

    Args:
        epsg (int): The number of the EPSG code

    Returns:
        tmp_loc (str): The name of the temporary location
        tmp_gisrc (str): The path to the original GISRC file
    """
    current_gisdbase = grass.gisenv()["GISDBASE"]
    srcgisrc = grass.tempfile()
    tmp_loc = f"temp_epsg{epsg}_location_{os.getpid()}"
    gisrc_file = open(srcgisrc, "w")
    gisrc_file.write("MAPSET: PERMANENT\n")
    gisrc_file.write(f"GISDBASE: {current_gisdbase}\n")
    gisrc_file.write(f"LOCATION_NAME: {tmp_loc}\n")
    gisrc_file.write("GUI: text\n")
    gisrc_file.close()

    proj_test = grass.parse_command("g.proj", flags="g")
    if "epsg" in proj_test:
        epsg_arg = {"epsg": epsg}
    else:
        epsg_arg = {"srid": f"EPSG:{epsg}"}
    # create temp location from input without import
    grass.verbose(_(f"Creating temporary location with EPSG:{epsg}..."))
    grass.run_command(
        "g.proj", flags="c", location=tmp_loc, quiet=True, **epsg_arg
    )

    # switch to temp location
    os.environ["GISRC"] = str(srcgisrc)
    proj = grass.parse_command("g.proj", flags="g")
    if "epsg" in proj:
        new_epsg = proj["epsg"]
    else:
        new_epsg = proj["srid"].split("EPSG:")[1]
    if new_epsg != str(epsg):
        grass.fatal(_("Creation of temporary location failed!"))

    return tmp_loc, srcgisrc


def get_current_location():
    """Get infos to current location

    Returns:
        loc (str): The name of the current location
        mapset (str): The name of the current mapset
        gisdbase (str): The current GISDBASE info
        gisrc (str): The path to the current GISRC file
    """
    # get current location, mapset, ...
    grassenv = grass.gisenv()
    loc = grassenv["LOCATION_NAME"]
    mapset = grassenv["MAPSET"]
    gisdbase = grassenv["GISDBASE"]
    gisrc = os.environ["GISRC"]
    return loc, mapset, gisdbase, gisrc


def switch_back_original_location(original_gisrc):
    """Switching back to original location after the computation in tmp
    location

    Args:
        original_gisrc (str): The path to the original GISRC file
    """
    # switch to target location
    os.environ["GISRC"] = str(original_gisrc)
